/*

	Filename:   ScrCheck.prg
   Version:    1.01
   Date:       2000/01/06

   dB/OS:      VdBase 7.x / Win95, Win98, Win98SE

   Author:     Ronnie MacGregor
   Contact:    RBMacG@High-Lugtonridge.co.uk
	
   
   Revisions:
   **********

   2000/03/04  Conditional use of icon and image changed to use the
               undocumented file('myfile',true) which allows this to
               work when these files are contained within an exe.  

   Description:
   ************

      ScrCheck() is designed to be called by your application on
      startup to check that the user's screen resolution and colour
      depth match or exceed your requirements.

      If the user's existing settings do not meet your requirements,
      then a form will open informing the user of this fact,
      showing them their existing settings, and the settings required.
      They then have the option to change their settings, continue
      unchanged, or quit, and the appropriate numerical code is 
      returned.
      
      If the user chooses to change their Display Properties, then
      ScrCheck() will open the Windows Display Properties Dialogue
      with the Settings tab already selected.

      ScrCheck() will return a numerical code depending on the
      screen settings found and if appropriate the users response:

        -1  Problem with passed parameters
         0  Present Screen settings OK
         1  Change Screen Display Properties
         2  Continue with existing settings
         3  Quit
         4  Windows Close "x" clicked

      The two main functions are self sufficient, and can be accessed
      directly along these lines:
      
         set procedure to ScrCheck.prg additive
         ResResult = GetScreenRes()
         ColResult = GetColourDepth()

      N.B. Watch out... We spell colour with a "u" <g>

   
   Syntax:
   *******

      ScrCheck(<expC>, <expN> [, <oRef>] )

      <expC>

         A String containing the required Screen Resolution
          - from this list:

         "640x480","800x600","1024x768","1152x864","1280x1024",
         "1600x1200"

      <expN>

         A number indicating the required Colour Depth
          - from this list:

         1,2,4,8,16,24,32

         1 Not specified, 2 bit Monochrome, 4 bit 16 Colours,
         8 bit 256 Colours, 16 bit 65,536 Colours, 24 bit True Colour,
         32 bit True Colour

      <oRef>

         Optional object reference, which if included and valid will
         mean that the following custom properties are attached to
         the referenced object

         <oRef>.ScreenRes  - A string containing the existing
                             screen resolution
                             e.g. "800x600"

         <oRef>.ColourDepth - A String containg the existing 
                              screen colour depth
                              e.g. "32 bit True Colour"


   Example:
   ********

      set procedure to ScrCheck.prg additive
      form.result = ScrCheck( "800x600", 24, form )

      or to handle the returned result:

      // Check display settings
      set procedure to ScrCheck.prg additive
      local nScrCheck
      nScrCheck = ScrCheck("800x600",16) // use 1600x1200 to test
      if nScrCheck <> 0
         do case
            case nScrCheck = -1 // Problem with passed parameters
               msgbox("Problem with passed parameters","Screen Check",16)
               quit
            case nScrCheck = 1 // Change Settings clicked
               msgbox("Please change your display settings in the" + chr(13) +;
                      "Windows dialogue that has just popped up, and" + chr(13) +;
                      "re-launch this application.","Screen Check",64)
               quit
            case nScrCheck = 2 // Continue unchanged clicked
               msgbox("Correct display may not be possible with your present ;
               screen settings.","Screen Check",16)
            case nScrCheck = 3 // Quit clicked
               msgbox("You have chosen to Quit.","Screen Check",16)
               quit
            case nScrCheck = 4 // Windows Close "x" clicked
               msgbox("The application will quit.","Screen Check",16)
               quit
         endcase
      endif
      release nScrCheck


   Dependancies:
   *************

      Monitor.ico  and  Monitor.jpg

      These should be found in the same directory as the .prg, and
      included in an exe, but everything has been coded in such a
      way that error free operation will be provided even if
      these files are not found.
      

   Acknowledgements:
   *****************

      The GetScreenRes() and GetColourDepth() functions are based on
      code from SysInfo.cc in the dUFLP.

      Thanks to Gary White, Romain Strieff, Ken Mayer and Ivar B Jessen
      for snippets of information given to me in the news groups
      which helped me to quickly finish this wee job. 
         
*/


function ScrCheck
   Parameters cRqdRes, cRqdDepth, oFormRef
   //
   // Check Resolution parameter
   do case
      case empty(argvector(1))
         msgbox("The calling program did not pass" + chr(13) + ;
                "the required Screen Resolution" + chr(13) + ;
                "parameter to ScrCheck","Call Error",16)
         return -1 // Error code
      case cRqdRes == "640x480"
      case cRqdRes == "800x600"
      case cRqdRes == "1024x768"
      case cRqdRes == "1152x864"
      case cRqdRes == "1280x1024"
      case cRqdRes == "1600x1200"
      otherwise
        msgbox("The calling program did not pass" + chr(13) + ;
               "a recognised Screen Resolution" + chr(13) + ;
               "parameter to ScrCheck","Call Error",16)
        return -1 // Error code
   endcase
   //
   // Check Colour Depth parameter
   do case
     case empty(argvector(2))
        msgbox("The calling program did not pass" + chr(13) + ;
               "the required Colour Depth" + chr(13) + ;
               "parameter to ScrCheck","Call Error",16)
        return -1 // Error code
     case cRqdDepth == 1
        cRqdDepth = '1 Not Specified'
     case cRqdDepth == 2
        cRqdDepth = '2 bit Monochrome'
     case cRqdDepth == 4
        cRqdDepth = '4 bit 16 Colours'
     case cRqdDepth == 8
        cRqdDepth = '8 bit 256 Colours'
     case cRqdDepth == 16
        cRqdDepth = '16 bit 65,536 Colours'
     case cRqdDepth == 24
        cRqdDepth = '24 bit True Colour'
     case cRqdDepth == 32
        cRqdDepth = '32 bit True Colour'
     otherwise
        msgbox("The calling program did not pass" + chr(13) + ;
               "a recognised Colour Depth" + chr(13) + ;
               "parameter to ScrCheck","Call Error",16)
        return -1 // Error code   
   endcase
   //
   // Check Flag parameter
   if not empty(argvector(3))
      try
         oFormRef.ScreenRes = GetScreenRes()
         oFormRef.ColourDepth = GetColourDepth()
      catch ( Exception e )
         msgbox("The calling program did not pass" + chr(13) + ;
               "a valid object reference" + chr(13) + ;
               "parameter to ScrCheck","Call Error",16)
         return -1 // Error code
      endTry
   endif
   //
   // Test existing settings OK
   // if not fire up dialogue form
   if val(cRqdRes) > val(GetScreenRes());
         or val(cRqdDepth) > val(GetColourDepth())
      // Fire up form
      local f, nReturn
      f=new ScreenResDialogueForm()
      f.readModal()
      nReturn = f.ReturnInfo 
      f.release()
      return nReturn   
   endif
// If we get to here then Current Settings OK
return 0 // OK
     

///////////////////////////////////      


class ScreenResDialogueForm of FORM
   with (this)
      onOpen = class::FORM_ONOPEN
      scaleFontBold = false
      metric = 6	// Pixels
      colorNormal = "silver"
      height = 364
      left = 167
      top = 150
      width = 517
      text = ""
      autoCenter = true
      mdi = false
      sizeable = false
      text = "  Display Settings Checker"
      sysMenu = true
      // Check for icon file first
      icon = iif(file('Monitor.ico',true),"filename Monitor.ico",null)
   endwith


   this.RECT_MONITOR = new RECTANGLE(this)
   with (this.RECT_MONITOR)
      left = 31
      top = 55
      width = 152
      height = 110
      text = ""
      colorNormal = "BtnText/lightsteelblue"
      border = false
      borderStyle = 4	// Single
   endwith


   this.PB_OPEN_DIALOGUE = new PUSHBUTTON(this)
   with (this.PB_OPEN_DIALOGUE)
      onClick = class::PB_OPEN_DIALOGUE_ONCLICK
      height = 25
      left = 82
      top = 275
      width = 354
      text = "Change my Display Properties ( recommended )"
      value = false
   endwith


   this.TXT_HORIZ = new TEXT(this)
   with (this.TXT_HORIZ)
      height = 13
      left = 95
      top = 6
      width = 54
      colorNormal = "blue/lightsteelblue"
      transparent = true
      fontBold = true
      text = "????"
   endwith


   this.TXT_VERT = new TEXT(this)
   with (this.TXT_VERT)
      height = 13
      left = 224
      top = 105
      width = 54
      colorNormal = "blue/lightsteelblue"
      transparent = true
      fontBold = true
      text = "????"
   endwith


   this.PB_CONTINUE = new PUSHBUTTON(this)
   with (this.PB_CONTINUE)
      onClick = class::PB_CONTINUE_ONCLICK
      height = 25
      left = 82
      top = 300
      width = 354
      text = "Continue with my current Display Properties"
      value = false
   endwith


   this.PB_QUIT = new PUSHBUTTON(this)
   with (this.PB_QUIT)
      onClick = class::PB_QUIT_ONCLICK
      height = 25
      left = 82
      top = 325
      width = 354
      text = "Quit"
      value = false
   endwith


   this.TXT_QUESTION = new TEXT(this)
   with (this.TXT_QUESTION)
      height = 17
      left = 167
      top = 250
      width = 185
      colorNormal = "BtnText/lightsteelblue"
      transparent = true
      fontSize = 11
      text = "What would you like to do ?"
   endwith


   this.TXT_COL_DEPTH_LABEL = new TEXT(this)
   with (this.TXT_COL_DEPTH_LABEL)
      height = 17
      left = 18
      top = 215
      width = 78
      colorNormal = "BtnText/lightsteelblue"
      transparent = true
      text = "Colour Depth"
   endwith


   this.TXT_COL_DEPTH = new TEXT(this)
   with (this.TXT_COL_DEPTH)
      height = 17
      left = 110
      top = 215
      width = 136
      colorNormal = "blue/lightsteelblue"
      transparent = true
      fontBold = true
      text = "Unknown"
   endwith


   this.RECTANGLE1 = new RECTANGLE(this)
   with (this.RECTANGLE1)
      left = 17
      top = 240
      width = 482
      height = 2
      text = ""
      colorNormal = "BtnText/lightsteelblue"
   endwith


   // Check for Image file first
   if file('Monitor.jpg',true)
      form.IMAGE1 = new IMAGE(this)
         with (form.IMAGE1)
         height = 188
         left = 14
         top = 20
         width = 205
         dataSource = 'FILENAME "Monitor.jpg"'
         alignment = 4	// True Size
         borderStyle = 3	// None
      endwith
   endif
   
   
   this.TEXT_PARA_1 = new TEXT(this)
   with (this.TEXT_PARA_1)
      height = 115
      left = 270
      top = 15
      width = 226
      colorNormal = "BtnText/lightsteelblue"
      transparent = true
      text = "Your current Screen Display Properties" + chr(13) + ;
             "are shown on the left." + chr(13) + chr(13) + ;
             "In order that this application displays" + chr(13) + ;
             "properly on the screen, you require" + chr(13) + ;
             "the following display settings," + chr(13) + ;
             "or higher :"
   endwith


   this.TXT_RQD_RES_LABEL = new TEXT(this)
   with (this.TXT_RQD_RES_LABEL)
      height = 22
      left = 270
      top = 135
      width = 150
      colorNormal = "BtnText/lightsteelblue"
      transparent = true
      text = "Required Resolution :"
   endwith


   this.TXT_RQD_DEPTH_LABEL = new TEXT(this)
   with (this.TXT_RQD_DEPTH_LABEL)
      height = 22
      left = 270
      top = 185
      width = 150
      colorNormal = "BtnText/lightsteelblue"
      transparent = true
      text = "Required Colour Depth :"
   endwith


   this.TXT_RQD_RES = new TEXT(this)
   with (this.TXT_RQD_RES)
      height = 22
      left = 297
      top = 160
      width = 150
      colorNormal = "red/lightsteelblue"
      transparent = true
      fontBold = true
      text = "???"
   endwith


   this.TXT_RQD_DEPTH = new TEXT(this)
   with (this.TXT_RQD_DEPTH)
      height = 22
      left = 297
      top = 210
      width = 150
      colorNormal = "red/lightsteelblue"
      transparent = true
      fontBold = true
      text = "???"
   endwith


   function form_onOpen
      // Initialise Return Info as quit so that we know
      // if form is closed via it's close box
      form.ReturnInfo = 4 // Windows close button
      // Display current settings 
      form.CurrScreenRes = GetScreenRes()
      form.Txt_Horiz.text = substr(form.CurrScreenRes,1,;
         (at("x",form.CurrScreenRes,1))-1)
      form.Txt_Vert.text = substr(form.CurrScreenRes,;
         (at("x",form.CurrScreenRes,1))+1,len(form.CurrScreenRes))
      form.Txt_Col_Depth.text = GetColourDepth()
      // Display required settings
      form.Txt_Rqd_Res.text = cRqdRes
      if cRqdDepth = "1 Not Specified"
         form.Txt_Rqd_Depth.text = substr(cRqdDepth,;
            (at(" ",cRqdDepth,1))+1,len(cRqdDepth))
      else
         form.Txt_Rqd_Depth.text = cRqdDepth
      endif
      //
      return

   
   function PB_CONTINUE_onClick
      form.ReturnInfo = 2 // Continue button clicked
      form.close()
      return


   function PB_OPEN_DIALOGUE_onClick
      // Open the Windows Display Settings Dialogue
      // Last parameter selects page:
      // 0  Background
      // 1  Screen Saver
      // 2  Appearance
      // 3  Settings
      run(true,"rundll32.exe shell32.dll,Control_RunDLL desk.cpl,,3")
      form.ReturnInfo = 1 // Change Properties button clicked
      form.close()
      return


   function PB_QUIT_onClick
      form.ReturnInfo = 3 // Quit button clicked
      form.close()
      return
endclass


///////////////////////////////////


function GetScreenRes
   local cScreenRes
   cScreenRes = ""
   if type('GetSystemMetrics') # 'FP'
      extern CINT GetSystemMetrics(CINT) user32
   endif
   try
      cScreenRes := getsystemmetrics(0)+'x'+getsystemmetrics(1)
   catch (exception e)
      cScreenRes := 'Undetermined'
   endtry
   //
   return cScreenRes


function GetColourDepth
   #define HANDLE       CHANDLE
   #define HDC          HANDLE
   #define LPCSTR       CSTRING
   #define LPSTRUCTURE  CPTR
   #define BITSPIXEL    12
   //
   if type('GetDeviceCaps') # 'FP'
      extern CINT GetDeviceCaps(HDC, CINT) gdi32
   endif
   if type('CreateDC') # 'FP'
      extern HDC CreateDC(LPCSTR, LPCSTR , LPCSTR , LPSTRUCTURE);
         gdi32 from 'CreateDCA'
   endif
   //
   try
      local cColorDepth, h, n
      cColorDepth = ""
      h = CreateDC('DISPLAY', '','','')
      n = GetDeviceCaps( h, BITSPIXEL )
      // ? n
      do case
         case n == 32
            cColorDepth = '32 bit True Colour'
         case n == 24
            cColorDepth = '24 bit True Colour'
         case n == 16
            cColorDepth = '16 bit 65,536 Colours'
         case n == 8
            cColorDepth = '8 bit 256 Colours'
         case n == 4
            cColorDepth = '4 bit 16 Colours'
         case n == 2
            cColorDepth = '2 bit Monochrome'
         otherwise
            cColorDepth = '0 bit Undetermined'
      endcase
   catch (exception e)
      cColorDepth := '0 bit Undetermined'
   endtry
   //
   release h, n
   return cColorDepth

//eop
